// Copyright (c) 2006-2007 nudge:nudge
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import <Cocoa/Cocoa.h>

typedef enum _NNQueueState {
	NNQueueEmpty = 0,
	NNQueueFilled = 1
} NNQueueState;

/**
\internal

original from http://www.cocoadev.com/index.pl?ProducersAndConsumerModel
 with some modifications/additions
 */
@interface NNQueue : NSObject {
	NSMutableArray* elements;
	NSConditionLock* lock; // 0 = no elements, 1 = elements
}

+ (NNQueue*)queue;

- (void)enqueue:(id)object;
- (void)enqueueObjects:(NSArray*)objects;
- (id)dequeue; // Blocks until there is an object to return

/**
blocks until a new object can be dequeued or the time ran out
 @param timeout time to wait for a new object
 @return object if timeout was not reached, nil otherwise
 */
- (id)dequeueWithTimeout:(NSTimeInterval)timeout;

/**
 @return nil if the queue is empty, otherwise the next element
 */
- (id)tryDequeue;

/**
empties the queue
 */
- (void)clear;

- (unsigned int)count;

@end
