// Copyright (c) 2006-2007 nudge:nudge
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import <Cocoa/Cocoa.h>
#import "NNTaggableObject.h"
#import "NSFileManager+TrashFile.h"
#import "NSFileManager+NNExtensions.h"

#include <stdio.h>
#include <string.h>
#include <sys/xattr.h>

extern NSString * const TAGGER_OPEN_COMMENT;
extern NSString * const TAGGER_CLOSE_COMMENT;
extern NSString * const TAGGER_WHITESPACE_SEPARATOR;

/**
Represents a file that can be tagged. Uses a subclass of NNTagReaderWriter to access the
 backing storage
 */
@interface NNFile : NNTaggableObject {
	
	NSString				*path;
	
	NSString				*kind;
	
	NSWorkspace				*workspace;
	NSFileManager			*fileManager;
}

/**
create a new NNFile from a path
 @param aPath path to file
 @returns NNFile representing file at aPath
 */
- (id)initWithPath:(NSString*)aPath;

/**
create a new NNFile from a path
 @param url needs to be a file url
 @returns NNFile for url
 */
- (id)initWithFileURL:(NSURL*)url;

/**
creates a file from a NSMetadataItem (returned by NSMetadataQuery)
 @param metadataItem
 @returns NNFile for metadataItem
 */
- (id)initWithNSMetadataItem:(NSMetadataItem*)metadataItem;

// convenience initializers

/**
create a new NNFile from a path
 @param aPath path to file
 @returns NNFile representing file at aPath
 */
+ (NNFile*)fileWithPath:(NSString*)aPath;

/**
create a new NSArray of NNFiles from a NSArray of file paths (NSString)
 @param filepaths paths to file (NSArray of NSString)
 @returns NSArray of NNFiles representing file at aPath
 */
+ (NSArray*)filesWithFilepaths:(NSArray*)filepaths;

/**
create a new NNFile from a path
 @param url needs to be a file url
 @returns NNFile for url
 */
+ (NNFile*)fileWithFileURL:(NSURL*)url;

/**
creates a file from a NSMetadataItem (returned by NSMetadataQuery)
 @param metadataItem
 @returns NNFile for metadataItem
 */
+ (NNFile*)fileWithNSMetadataItem:(NSMetadataItem*)metadataItem;

// file wrapping stuff

- (NSString*)path;						/**< full path including file.extension */
- (NSString*)standardizedPath;			/**< standardized path */
- (NSString*)filename;					/**< file name including extension */
- (NSString*)extension;					/**< file extension */
- (NSString*)directory;					/**< directory path the file is located in */
- (BOOL)isDirectory;					/**< checks if file is directory */
- (NSImage*)icon;						/**< icon representing file */
- (NSString *)kind;						/**< A description of the kind of item this file represents */

- (NSDate *)creationDate;				/**< Indicates the file's creation date */
- (NSDate *)modificationDate;			/**< Indicates the file's modification date */
- (unsigned long long)size;				/**< Indicates the file's logical size in bytes (data + resource fork). Does not work for folders. */

// Compatibility
- (id)valueForAttribute:(id)attribute;

@end
