// Copyright (c) 2006-2007 nudge:nudge
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#import <Cocoa/Cocoa.h>
#import "NNTag.h"

/**
container class for selected tags
 */
@interface NNSelectedTags : NSObject {
	NSMutableArray *selectedTags;
	NNTag *lastTag;
	
	NSNotificationCenter *nc;
}

/**
directly set some NNTags as selected tags
 @param tags tags to set
 @returns prepared selected tags
 */
- (id)initWithTags:(NSArray*)tags;

/**
@returns currently selected NNTags
 */
- (NSMutableArray*)selectedTags;

/**
@param otherTags tags to set
 */
- (void)setSelectedTags:(NSArray*)otherTags;

/**
removes most recently added selected tag
 */
- (void)removeLastTag;

/**
empties selected tags
 */
- (void)removeAllTags;

/**
@returns number of selected tags
 */
- (unsigned int)count;

/**
@param aTag NNTag to add
 adds a tag to selected tags
 */
- (void)addTag:(NNTag*)aTag;

/**
@param someTags NSArray of NNTag objects to add
 adds all objects from someTags to selected tags 
 */
- (void)addTags:(NSArray*)someTags;

/**
@param aTag NNTag to remove
 aTag will be removed (if present)
 */
- (void)removeTag:(NNTag*)aTag;

/**
@param aTag NNTag to look for
 @returns YES if selected tags contain aTag; NO otherwise
 */
- (BOOL)containsTag:(NNTag*)aTag;

/**
@returns enumerator for selected tags
 */
- (NSEnumerator*)objectEnumerator;

/**
convenience method to add multiple NNTags
 @param array NSArray of NNTags to add
 */
- (void)addObjectsFromArray:(NSArray*)array;

/** 
convenience method to remove multiple NNTags
@param array NSArray of NNTags to remove
*/
- (void)removeObjectsInArray:(NSArray*)array;

@end